package com.qmth.teachcloud.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.OrgCenterTypeEnum;
import com.qmth.teachcloud.common.service.BasicSchoolService;
import com.qmth.teachcloud.common.service.CallApiOrgCenterService;
import com.qmth.teachcloud.common.service.OrgCenterDataDisposeService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 机构数据中心数据处理服务实现类
 * @Author: CaoZixuan
 * @Date: 2021-04-02
 */
@Service
public class OrgCenterDataDisposeServiceImpl implements OrgCenterDataDisposeService {
    @Resource
    private CallApiOrgCenterService callApiOrgCenterService;
    @Resource
    private BasicSchoolService basicSchoolService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateSchoolInfo() throws IOException {
        List<Map> result = callApiOrgCenterService.callOrgInfo();
        if (Objects.isNull(result) || result.size() == 0) {
            return;
        }
        // 禁用原有学校
        List<BasicSchool> basicSchoolList = basicSchoolService.list();
        basicSchoolService.update(new UpdateWrapper<BasicSchool>().lambda().set(BasicSchool::getEnable, false).gt(BasicSchool::getId,0));


        for (Map map : result) {
            String code = String.valueOf(map.get("code"));
            String name = String.valueOf(map.get("name"));
            String logo = String.valueOf(map.get("logo"));
            String accessKey = String.valueOf(map.get("accessKey"));
            String accessSecret = String.valueOf(map.get("accessSecret"));
            Map typeMap = JSONObject.parseObject(String.valueOf(map.get("type")));
            String typeCode = String.valueOf(typeMap.get("code"));
            String typeName = String.valueOf(typeMap.get("name"));

            List<BasicSchool> oldSchoolList = basicSchoolList.stream().filter(e -> e.getCode().equals(code)).collect(Collectors.toList());
            if (oldSchoolList.size() > 1){
                throw ExceptionResultEnum.ERROR.exception("学校信息异常");
            }
            Long schoolId;
            if (oldSchoolList.size() > 0){
                // 系统中原先就有该学校 -》 更新
                BasicSchool oldSchool = oldSchoolList.get(0);
                schoolId = oldSchool.getId();
            }else {
                // 系统中没有该学校 -》 新增 (系统中如果id被占用会报错)
                schoolId = SystemConstant.getDbUuid();
            }

            BasicSchool school = new BasicSchool();
            school.setId(schoolId);
            school.setCode(code);
            school.setName(name);
            school.setEnable(true);
            school.setAccessKey(accessKey);
            school.setAccessSecret(accessSecret);
            school.setLogo(logo);
            basicSchoolService.saveOrUpdate(school);
        }
    }
}
